<?php
require_once('jwt-adumo.php');
class ControllerExtensionPaymentAdumo extends Controller {
	public function index() {
		$data['button_confirm'] = $this->language->get('button_confirm');

		$this->load->model('checkout/order');
        $this->load->model('account/customer');
        $this->load->model('extension/payment/adumo');

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $data['flow'] = '';
        $data['puid']="";
		$data['MerchantID'] = $this->config->get('adumo_merchant_id');
        $data['ApplicationID'] = $this->config->get('adumo_application_id');
        $data['MerchantReference'] = $this->session->data['order_id'];
        $data['Amount'] = $order_info['total'];
        $data['CurrencyCode'] = $order_info['currency_code'];

        if($this->config->get('adumo_merchant_suffix'))
            $data['MerchantReference'] = $data['MerchantReference'] ."_". $this->config->get('adumo_merchant_suffix');
        
        /**
		 * load the language file
		 */
		$this->load->language('extension/payment/adumo');
        $data['Action'] = $this->config->get('adumo_merchant_payment_url');
        $data['PayNowButton'] = $this->language->get('payment_button');

		if ($this->cart->hasShipping()) {
			$data['Recipient'] = $order_info['shipping_firstname'] . ' ' . $order_info['shipping_lastname'];
			$data['ShippingAddress1'] = $order_info['shipping_address_1'];
			$data['ShippingAddress2'] = $order_info['shipping_address_2'];
			$data['ShippingAddress3'] = $order_info['shipping_city'];
			$data['ShippingAddress4'] = $order_info['shipping_postcode'];
			$data['ShippingAddress5'] = $order_info['shipping_country'];
		} else {
			$data['Recipient'] = '';
			$data['ShippingAddress1'] = '';
			$data['ShippingAddress2'] = '';
			$data['ShippingAddress3'] = '';
			$data['ShippingAddress4'] = '';
			$data['ShippingAddress5'] = '';
		}

		$data['currency'] = $this->session->data['currency'];
		$data['RedirectSuccessfulURL'] = $this->url->link('extension/payment/adumo/callback', '', true);
        $data['RedirectFailedURL'] = $this->url->link('extension/payment/adumo/callback', '', true);
        $payload = array(
            'iss'=>$this->language->get('payment_issuer'),
            'cuid'=>$data['MerchantID'],
            'auid' => $data['ApplicationID'],
            'amount'=>$data['Amount'],
            'mref'=>$data['MerchantReference'],
            'jti'=>base64_encode(random_bytes (32)),
            'iat'=>time() - 60,
            'exp'=>time() + 43200 /** change to the expiration to 12hr was requested */
        );

        /**
         * check if the customer is logged in
         */
        if($this->customer->isLogged()){
            $customer_id = $this->customer->getId();
            $payment_token = $this->model_extension_payment_adumo->getPaymentToken($customer_id);
            /**
             * if they have a payment token then attach it
             */
            if($payment_token)
                $data['puid']=$payment_token;
        }

        $data['Token'] = JWT::encode($payload,$this->config->get('adumo_merchant_secret'));

		return $this->load->view('extension/payment/adumo', $data);
	}

    public function webhook(){
        if($_SERVER['CONTENT_TYPE']=="application/json"){
            /**
             * load the checkout model
             * load the customer model
             * load the Adumo model
             */
            $this->load->model('checkout/order');
            $this->load->model('account/customer');
            $this->load->model('extension/payment/adumo');
            
            $data = json_decode(trim(file_get_contents('php://input')),true);
            $order_id = $data['merchantReference'];

            if($order_id && $this->config->get('adumo_merchant_suffix'))
                $order_id = explode("_",$order_id)[0];

            $order_info = $this->model_checkout_order->getOrder($order_id);

            if(
                $order_info /** check if the order ID exists and returned real data */
                && $data['status'] == "SETTLED"  /** check if the transaction was completed successfully */
                && $order_info['order_status_id'] != $this->config->get('adumo_order_status_id') /** check that the order has not already been updated */
            ){
                /**
                 * verify the _RESPONSE_TOKEN from Adumo
                 */
                try{
                    $payload = JWT::decode($data['token'],$this->config->get('adumo_merchant_secret'),array('HS256'));

                    if(
                        $payload->{'status'}==$data['status'] /** check if the results match */
                        && $payload->{'puid'}==$data['puid'] /** check that the PUID match */
                        && $payload->{'transactionIndex'} == $data['transactionId'] /** check if the transaction indexes match */
                    ){
                        if($order_info['customer_id']){
                            /** 
                             * Save the payment token and the payment method. Blank the payment method
                             * if the payment token is empty
                             * 
                             * 
                             * NB: A guest customer will have a 'customer_id' of 0 in the `oc_order` table
                             * but it is not the case if you get the ID from '$this->customer->getId()'
                             */
                            $this->model_extension_payment_adumo->addPaymentToken($order_info['customer_id'],$data['puid'], $data['puid'] ? $data['paymentMethod'] : "");
                        }

                        /**
                         * update the checkout to the extension's order status 
                         */
                        $this->model_checkout_order->addOrderHistory( $order_id, $this->config->get('adumo_order_status_id'));
                    }
                }
                catch(Exception $e){}
            }
        }
    }

	public function callback(){
        /**
         * load the checkout model
         * load the customer model
         * load the Adumo model
         */
        $this->load->model('checkout/order');
        $this->load->model('account/customer');
        $this->load->model('extension/payment/adumo');

        $order_id = $this->request->post['_MERCHANTREFERENCE'];

        if($order_id && $this->config->get('adumo_merchant_suffix'))
            $order_id = explode("_",$order_id)[0];

        $order_info = $this->model_checkout_order->getOrder($order_id);

        
        if(
            $order_info /** check if the order ID exists and returned real data */
            && ($this->request->post['_RESULT'] == 0 || $this->request->post['_RESULT'] == 1) /** check if the transaction was completed successfully */
        ){
            /**
             * verify the _RESPONSE_TOKEN from Adumo
             */
            try{
                $payload = JWT::decode($this->request->post['_RESPONSE_TOKEN'],$this->config->get('adumo_merchant_secret'),array('HS256'));

                if(
                    $payload->{'result'}==$this->request->post['_RESULT'] /** check if the results match */ 
                    && $payload->{'transactionIndex'} == $this->request->post['_TRANSACTIONINDEX'] /** check if the transaction indexes match */
                ){
                    if( $order_info['order_status_id'] != $this->config->get('adumo_order_status_id')){ /** check that the order has not already been updated */
                        if($order_info['customer_id']){
                            /** 
                             * Save the payment token and the payment method. Blank the payment method
                             * if the payment token is empty
                             * 
                             * 
                             * NB: A guest customer will have a 'customer_id' of 0 in the `oc_order` table
                             * but it is not the case if you get the ID from '$this->customer->getId()'
                             */
                            $this->model_extension_payment_adumo->addPaymentToken($order_info['customer_id'],$this->request->post['puid'], $this->request->post['puid'] ? $this->request->post['_PAYMETHOD'] : "");
                        }

                        /** 
                         * update the checkout to the extension's order status 
                         */
                        $this->model_checkout_order->addOrderHistory( $order_id, $this->config->get('adumo_order_status_id')); 
                    }

                    /** 
                     * redirect to successful checkout URL 
                     */
                    $this->response->redirect($this->url->link('checkout/success')); 
                }
                else{
                    $this->response->redirect($this->url->link('checkout/failure')); /** redirect to failed checkout URL */
                }
            }
            catch(Exception $e){
                $this->response->redirect($this->url->link('checkout/failure')); /** redirect to failed checkout URL */
            }
        }
        else{
            $this->response->redirect($this->url->link('checkout/failure')); /** redirect to failed checkout URL */
        }
	}
}